"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _config = _interopRequireDefault(require("../config"));

var _GitHub = _interopRequireDefault(require("../lib/GitHub"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _default = async (req, res) => {
  const ua = _config.default.get('analytics.uaTrackingId') ? require('universal-analytics')(_config.default.get('analytics.uaTrackingId')) : null;
  const github = await new _GitHub.default({
    username: req.params.username,
    repository: req.params.repository,
    branch: req.params.branch,
    token: _config.default.get('githubToken'),
    version: req.params.version
  });

  const isAppAuth = _config.default.get('githubAppID') && _config.default.get('githubPrivateKey');

  if (isAppAuth) {
    return res.send('Staticman connected!');
  }

  return github.api.repos.listInvitationsForAuthenticatedUser({}).then(({
    data
  }) => {
    const expectedRepoName = `${req.params.username}/${req.params.repository}`;
    const collaborationInvite = Array.isArray(data) && data.find(invitation => invitation.repository.full_name === expectedRepoName);

    if (!collaborationInvite) {
      return res.status(404).send('Invitation not found');
    }

    return github.api.repos.acceptInvitation({
      invitation_id: collaborationInvite.id
    }).then(() => {
      res.send('Staticman connected!');

      if (ua) {
        ua.event('Repositories', 'Connect').send();
      }
    }).catch(() => {
      res.status(500).send('Error');

      if (ua) {
        ua.event('Repositories', 'Connect error').send();
      }
    });
  });
};

exports.default = _default;