"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _config = _interopRequireDefault(require("../config"));

var _GitHub = _interopRequireDefault(require("../lib/GitHub"));

var _Staticman = _interopRequireDefault(require("../lib/Staticman"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _default = async (repo, data) => {
  const ua = _config.default.get('analytics.uaTrackingId') ? require('universal-analytics')(_config.default.get('analytics.uaTrackingId')) : null;

  if (!data.number) {
    return;
  }

  const github = await new _GitHub.default({
    username: data.repository.owner.login,
    repository: data.repository.name,
    version: '1'
  });

  try {
    const review = await github.getReview(data.number);

    if (review.sourceBranch.indexOf('staticman_')) {
      return;
    }

    if (review.state !== 'merged' && review.state !== 'closed') {
      return;
    }

    if (review.state === 'merged') {
      const bodyMatch = review.body.match(/(?:.*?)<!--staticman_notification:(.+?)-->(?:.*?)/i);

      if ((bodyMatch === null || bodyMatch === void 0 ? void 0 : bodyMatch.length) === 2) {
        try {
          const parsedBody = JSON.parse(bodyMatch[1]);
          const staticman = await new _Staticman.default(parsedBody.parameters);
          staticman.setConfigPath(parsedBody.configPath);
          staticman.processMerge(parsedBody.fields, parsedBody.options);
        } catch (err) {
          console.log(err);

          if (ua) {
            ua.event('Hooks', 'Process merge error').send();
          }
        }
      }
    }

    if (ua) {
      ua.event('Hooks', 'Delete branch').send();
    }

    github.deleteBranch(review.sourceBranch);
  } catch (err) {
    console.log(err);

    if (ua) {
      ua.event('Hooks', 'Delete branch error').send();
    }
  }
};

exports.default = _default;