"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createConfigObject = createConfigObject;
exports.processEntry = processEntry;
exports.sendResponse = sendResponse;
exports.default = void 0;

var _ReCaptcha = _interopRequireDefault(require("../lib/ReCaptcha"));

var _ErrorHandler = require("../lib/ErrorHandler");

var _Staticman = _interopRequireDefault(require("../lib/Staticman"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function createConfigObject(apiVersion, property) {
  const remoteConfig = {};

  if (apiVersion === '1') {
    remoteConfig.file = '_config.yml';
    remoteConfig.path = 'staticman';
  } else {
    remoteConfig.file = 'staticman.yml';
    remoteConfig.path = property || '';
  }

  return remoteConfig;
}

async function processEntry(staticman, req, res) {
  const fields = req.query.fields || req.body.fields;
  const options = req.query.options || req.body.options;
  const data = await staticman.processEntry(fields, options);
  sendResponse(res, {
    redirect: data.redirect,
    fields: data.fields
  });
}

function sendResponse(res, data) {
  const error = data === null || data === void 0 ? void 0 : data.err;
  const statusCode = error ? 500 : 200;

  if (!error && data.redirect) {
    return res.redirect(data.redirect);
  }

  if (error && data.redirectError) {
    return res.redirect(data.redirectError);
  }

  const payload = {
    success: !error
  };

  if (error !== null && error !== void 0 && error._smErrorCode) {
    const errorCode = (0, _ErrorHandler.getInstance)().getErrorCode(error._smErrorCode);
    const errorMessage = (0, _ErrorHandler.getInstance)().getMessage(error._smErrorCode);

    if (errorMessage) {
      payload.message = errorMessage;
    }

    if (error.data) {
      payload.data = error.data;
    }

    if (error) {
      payload.rawError = error;
    }

    payload.errorCode = errorCode;
  } else if (error) {
    payload.rawError = data.err.toString();
  } else {
    payload.fields = data.fields;
  }

  return res.status(statusCode).send(payload);
}

var _default = async (req, res) => {
  const staticman = await new _Staticman.default(req.params, {
    ip: req.headers['x-forwarded-for'] || req.connection.remoteAddress,
    userAgent: req.headers['user-agent']
  });

  try {
    await (0, _ReCaptcha.default)(staticman, req);
    processEntry(staticman, req, res);
  } catch (err) {
    var _req$body, _req$body$options, _req$body2, _req$body2$options;

    sendResponse(res, {
      err,
      redirect: req === null || req === void 0 ? void 0 : (_req$body = req.body) === null || _req$body === void 0 ? void 0 : (_req$body$options = _req$body.options) === null || _req$body$options === void 0 ? void 0 : _req$body$options.redirect,
      redirectError: req === null || req === void 0 ? void 0 : (_req$body2 = req.body) === null || _req$body2 === void 0 ? void 0 : (_req$body2$options = _req$body2.options) === null || _req$body2$options === void 0 ? void 0 : _req$body2$options.redirectError
    });
  }
};

exports.default = _default;