"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.requestGitLabAccessToken = exports.requestGitHubAccessToken = void 0;

var _requestPromise = _interopRequireDefault(require("request-promise"));

var _config = _interopRequireDefault(require("../config"));

var _ErrorHandler = _interopRequireDefault(require("./ErrorHandler"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const requestGitHubAccessToken = (code, clientId, clientSecret, redirectUri) => {
  return (0, _requestPromise.default)({
    headers: {
      Accept: 'application/json'
    },
    json: true,
    method: 'POST',
    uri: _config.default.get('githubAccessTokenUri'),
    qs: {
      code,
      client_id: clientId,
      client_secret: clientSecret,
      redirect_uri: redirectUri
    }
  }).then(res => res.access_token).catch(err => Promise.reject((0, _ErrorHandler.default)('GITHUB_AUTH_FAILED', {
    err
  }))); // eslint-disable-line handle-callback-err
};

exports.requestGitHubAccessToken = requestGitHubAccessToken;

const requestGitLabAccessToken = (code, clientId, clientSecret, redirectUri) => {
  return (0, _requestPromise.default)({
    headers: {
      Accept: 'application/json'
    },
    json: true,
    method: 'POST',
    uri: _config.default.get('gitlabAccessTokenUri'),
    qs: {
      code,
      client_id: clientId,
      client_secret: clientSecret,
      grant_type: 'authorization_code',
      redirect_uri: redirectUri
    }
  }).then(res => res.access_token).catch(err => Promise.reject((0, _ErrorHandler.default)('GITLAB_AUTH_FAILED', {
    err
  }))); // eslint-disable-line handle-callback-err
};

exports.requestGitLabAccessToken = requestGitLabAccessToken;