"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = checkRecaptcha;

var _expressRecaptcha = _interopRequireDefault(require("express-recaptcha"));

var _ErrorHandler = _interopRequireDefault(require("./ErrorHandler"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function checkRecaptcha(staticman, req) {
  return new Promise((resolve, reject) => {
    staticman.getSiteConfig().then(siteConfig => {
      var _req$body, _req$body$options;

      if (!siteConfig.get('reCaptcha.enabled')) {
        return resolve(false);
      }

      const reCaptchaOptions = req === null || req === void 0 ? void 0 : (_req$body = req.body) === null || _req$body === void 0 ? void 0 : (_req$body$options = _req$body.options) === null || _req$body$options === void 0 ? void 0 : _req$body$options.reCaptcha;

      if (!reCaptchaOptions || !reCaptchaOptions.siteKey || !reCaptchaOptions.secret) {
        return reject((0, _ErrorHandler.default)('RECAPTCHA_MISSING_CREDENTIALS'));
      }

      let decryptedSecret;

      try {
        decryptedSecret = staticman.decrypt(reCaptchaOptions.secret);
      } catch (err) {
        return reject((0, _ErrorHandler.default)('RECAPTCHA_CONFIG_MISMATCH'));
      }

      if (reCaptchaOptions.siteKey !== siteConfig.get('reCaptcha.siteKey') || decryptedSecret !== siteConfig.get('reCaptcha.secret')) {
        return reject((0, _ErrorHandler.default)('RECAPTCHA_CONFIG_MISMATCH'));
      }

      _expressRecaptcha.default.init(reCaptchaOptions.siteKey, decryptedSecret);

      _expressRecaptcha.default.verify(req, () => {
        var _req$recaptcha;

        return req !== null && req !== void 0 && (_req$recaptcha = req.recaptcha) !== null && _req$recaptcha !== void 0 && _req$recaptcha.error ? reject((0, _ErrorHandler.default)(req.reCaptcha.error)) : resolve(true);
      });

      return resolve(true);
    }).catch(err => reject(err));
  });
}